using UnityEngine;

namespace Mod
{
    public class Mod
    {
        public static Sprite sleepSprite = ModAPI.LoadSprite("sleep.png");
        public static Sprite deadSprite = ModAPI.LoadSprite("dead.png");
        public static Sprite bleedSprite = ModAPI.LoadSprite("bleed.png");
        public static Sprite bloodSprite = ModAPI.LoadSprite("bloodloss.png");
        public static Sprite ozxgenSprite = ModAPI.LoadSprite("oxygen.png");
        public static Sprite oxygenCritSprite = ModAPI.LoadSprite("oxygencritical.png");
        public static Sprite painSprite = ModAPI.LoadSprite("shock.png");
        public static Sprite fireSprite = ModAPI.LoadSprite("fire.png");
        public static Sprite coldSprite = ModAPI.LoadSprite("cold.png");
        public static Sprite hpSprite = ModAPI.LoadSprite("lowhp.png");
        public static Sprite intBleedSprite = ModAPI.LoadSprite("intbleed.png");
        public static bool onlyDetailView;
        public static void Main()
        {
            
            ModAPI.OnItemSpawned += (sender, obj) => {
                if(obj.Instance.GetComponent<PersonBehaviour>())
                {
                    obj.Instance.GetOrAddComponent<StatusIcons>();
                }
            };
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"), //item to derive from
                   NameOverride = "Detail view toggle", //new item name with a suffix to assure it is globally unique
                   DescriptionOverride = "When active, status icons will only be shown when detail view is on (Default: S).", //new item description
                   CategoryOverride = ModAPI.FindCategory("Entities"), //new item category
                  ThumbnailOverride = ModAPI.LoadSprite("detailtoggle.png"), //new item thumbnail (relative path)
                  AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                  {
                      Instance.GetComponent<PhysicalBehaviour>().Disintegrate();
                      UnityEngine.Object.Destroy(Instance, 3f); //wait for destruction so scripts dont break
                      onlyDetailView = !onlyDetailView;
                      if(onlyDetailView)
                      {
                          ModAPI.Notify("Status Icons will now only be shown in Detail View.");
                      }
                      else
                      {
                          ModAPI.Notify("Status Icons will now be shown at all times.");
                      }
                  }
                }
            );
        }
    }
    public class StatusIcons : MonoBehaviour
    {
        [SkipSerialisation]
        public LimbBehaviour head;
        [SkipSerialisation]
        public PersonBehaviour person;
        [SkipSerialisation]
        public int statusNumber;
        [SkipSerialisation]
        public bool[] statuses = new bool[11]; //goes like this: CONSCIOUSNESS, DEATH, BLEEDING, BLOOD LOSS, OXYGEN LOW, OXYGEN CRITICAL, SHOCK, FIRE, COLD, LOWHP, INTERNALBLEED
        [SkipSerialisation]
        public GameObject[] statusObjects = new GameObject[11];
        [SkipSerialisation]
        public float timeUntilCheck;
        void Awake()
        {
            person = GetComponent<PersonBehaviour>();
            foreach(LimbBehaviour limb in person.Limbs)
            {
                if(limb.RoughClassification == LimbBehaviour.BodyPart.Head)
                {
                    head = limb;
                    return;
                }
            }
        }
        void Update()
        {
            timeUntilCheck += Time.deltaTime;
            if(timeUntilCheck > 1f)
            {
                timeUntilCheck = 0f;
                UpdateStatus();
            }
            int indexed = 0;
            for (int i = 0; i < statusObjects.Length; i++)
            {
                if(statusObjects[i])
                {
                    statusObjects[i].transform.position = new Vector3(head.transform.position.x + (indexed - (statusNumber - 1) * 0.5f) * 0.175f, head.transform.position.y + 0.35f, 0f);
                    indexed += 1;
                }
            }
        }
        void UpdateStatus()
        {
            statusNumber = 0;
            foreach(GameObject obj in statusObjects)
            {
                if(obj)
                {
                    Destroy(obj);
                }
            }
            for (int i = 0; i < statuses.Length; i++)
            {
                statuses[i] = false;
            }
            if (Mod.onlyDetailView && Global.main.ShowLimbStatus || !Mod.onlyDetailView)
            {
                if (!person.IsAlive())
                {
                    statuses[1] = true;
                    statusNumber += 1;
                }
                else
                {
                    if (person.Consciousness < 0.8f || head.Numbness > 1f)
                    {
                        statuses[0] = true;
                        statusNumber += 1;
                    }
                    bool bleeding = false;
                    foreach (LimbBehaviour limb in person.Limbs)
                    {
                        if (limb.CirculationBehaviour.BleedingRate > 0.1f)
                        {
                            bleeding = true;
                        }
                    }
                    if (bleeding) statusNumber += 1;
                    statuses[2] = bleeding;
                    int intbleed = 0;
                    foreach (LimbBehaviour limb in person.Limbs)
                    {
                        if (limb.CirculationBehaviour.InternalBleedingIntensity > 0.2f)
                        {
                            intbleed += 1;
                        }
                    }
                    if (intbleed >= 3) { statusNumber += 1; statuses[10] = true; }
                    if (head.CirculationBehaviour.GetAmountOfBlood() < 0.9f) { statuses[3] = true; statusNumber += 1; }
                    if (person.OxygenLevel < 0.9f && person.OxygenLevel > 0f) { statuses[4] = true; statusNumber += 1; }
                    if (person.OxygenLevel <= 0f) { statuses[5] = true; statusNumber += 1; }
                    if (person.ShockLevel > 0.4f || person.PainLevel > 0.35f) { statuses[6] = true; statusNumber += 1; }
                    if (head.GetComponent<PhysicalBehaviour>().Temperature > head.DiscomfortingHeatTemperature) { statuses[7] = true; statusNumber += 1; }
                    if (head.GetComponent<PhysicalBehaviour>().Temperature < head.FreezingTemperature) { statuses[8] = true; statusNumber += 1; }
                    if (head.Health / head.InitialHealth < 0.25f) { statuses[9] = true; statusNumber += 1; }
                }
            }

            for (int i = 0; i < statuses.Length; i++)
            {
                if (statuses[i])
                {
                    GameObject g = new GameObject();
                    g.transform.SetParent(transform);
                    SpriteRenderer spr = g.AddComponent<SpriteRenderer>();
                    g.transform.position = head.transform.position;
                    g.transform.localScale = new Vector3(0.5f, 0.5f, 1f);
                    g.layer = 16;
                    g.AddComponent<Optout>();
                    spr.color = new Color(1f, 1f, 1f, 0.7f);
                    statusObjects[i] = g;
                    switch (i)
                    {
                        case 0:
                            spr.sprite = Mod.sleepSprite;
                            break;
                        case 1:
                            spr.sprite = Mod.deadSprite;
                            break;
                        case 2:
                            spr.sprite = Mod.bleedSprite;
                            break;
                        case 3:
                            spr.sprite = Mod.bloodSprite;
                            break;
                        case 4:
                            spr.sprite = Mod.ozxgenSprite;
                            break;
                        case 5:
                            spr.sprite = Mod.oxygenCritSprite;
                            break;
                        case 6:
                            spr.sprite = Mod.painSprite;
                            break;
                        case 7:
                            spr.sprite = Mod.fireSprite;
                            break;
                        case 8:
                            spr.sprite = Mod.coldSprite;
                            break;
                        case 9:
                            spr.sprite = Mod.hpSprite;
                            break;
                        case 10:
                            spr.sprite = Mod.intBleedSprite;
                            break;
                    }
                }
            }
        }
    }
}